@@ -1,36 +1,38 @@
-/* radare - LGPL - Copyright 2009-2021 - pancake */
+/* radare - LGPL - Copyright 2009-2022 - pancake */
 
 #ifndef R2_CORE_H
 #define R2_CORE_H
 
 #include <r_main.h>
-#include "r_socket.h"
-#include "r_types.h"
-#include "r_magic.h"
-#include "r_agraph.h"
-#include "r_io.h"
-#include "r_fs.h"
-#include "r_lib.h"
-#include "r_diff.h"
-#include "r_egg.h"
-#include "r_lang.h"
-#include "r_asm.h"
-#include "r_parse.h"
-#include "r_anal.h"
-#include "r_cmd.h"
-#include "r_cons.h"
-#include "r_search.h"
-#include "r_sign.h"
-#include "r_debug.h"
-#include "r_flag.h"
-#include "r_config.h"
-#include "r_bin.h"
-#include "r_hash.h"
-#include "r_util.h"
-#include "r_util/r_print.h"
-#include "r_crypto.h"
-#include "r_bind.h"
-#include "r_codemeta.h"
+#include <r_socket.h>
+#include <r_types.h>
+#include <r_magic.h>
+#include <r_agraph.h>
+#include <r_io.h>
+#include <r_fs.h>
+#include <r_lib.h>
+#include <r_egg.h>
+#include <r_lang.h>
+#include <r_asm.h>
+#include <r_parse.h>
+#include <r_anal.h>
+#include <r_cmd.h>
+#include <r_cons.h>
+#include <r_search.h>
+#include <r_sign.h>
+#include <r_debug.h>
+#include <r_flag.h>
+#include <r_config.h>
+#include <r_bin.h>
+#include <r_hash.h>
+#include <r_util.h>
+#include <r_util/r_print.h>
+#include <r_crypto.h>
+#include <r_bind.h>
+#include <r_codemeta.h>
+
+// TODO: thois var should be 1 at some point :D
+#define SHELLFILTER 0
 
 #ifdef __cplusplus
 extern "C" {
@@ -218,7 +220,6 @@ typedef struct r_core_visual_t {
 	RList *tabs;
 	int tab;
 } RCoreVisual;
-// #define RCoreVisual Visual
 
 typedef struct {
 	int x;
@@ -243,9 +244,21 @@ typedef struct r_core_tasks_t {
 	bool oneshot_running;
 } RCoreTaskScheduler;
 
+typedef enum {
+	VC_RVC,
+	VC_GIT
+} VcType;
+
+typedef struct {
+	char *path;
+	VcType type;
+	Sdb *db;
+} Rvc;
+
 typedef struct r_core_project_t {
 	char *name;
 	char *path;
+	Rvc *rvc;
 } RProject;
 
 R_API RProject *r_project_new(void);
@@ -256,7 +269,7 @@ R_API bool r_project_open(RProject *p, const char *prjname, const char *path);
 R_API void r_project_save(RProject *p);
 R_API void r_project_free(RProject *p);
 R_API bool r_project_is_loaded(RProject *p);
-R_API bool r_core_project_is_saved(RCore *core);
+R_API bool r_core_project_is_dirty(RCore *core);
 
 struct r_core_t {
 	RBin *bin;
@@ -270,8 +283,8 @@ struct r_core_t {
 	RBuffer *yank_buf;
 	ut64 yank_addr;
 	bool tmpseek;
-	bool vmode;
-	int interrupted; // XXX IS THIS DUPPED SOMEWHERE?
+	bool vmode; // is r2 in visual or panels mode?
+	int interrupted; // XXX R2_580 - this variable is unused and must be removed
 	/* files */
 	RCons *cons;
 	RIO *io;
@@ -314,7 +327,7 @@ struct r_core_t {
 	bool keep_asmqjmps;
 	RCoreVisual visual;
 	// visual // TODO: move them into RCoreVisual
-	int http_up;
+	int http_up; // R2_580 bool
 	int gdbserver_up;
 	RCoreVisualMode printidx;
 	char *stkcmd;
@@ -333,6 +346,9 @@ struct r_core_t {
 	char *cmdfilter;
 	bool break_loop;
 	RList *undos;
+#if R2_580
+	int undoindex;
+#endif
 	bool binat;
 	bool fixedbits; // will be true when using @b:
 	bool fixedarch; // will be true when using @a:
@@ -350,13 +366,18 @@ struct r_core_t {
 	bool log_events; // core.c:cb_event_handler : log actions from events if cfg.log.events is set
 	RList *ropchain;
 	char *theme;
-	int in_cmdstr;
+	char *themepath;
+	bool allbins;
 	bool marks_init;
 	ut64 marks[UT8_MAX + 1];
-
+	RThreadChannel *chan; // query
+#if R2_580
+	bool in_log_process // false;
+#endif
 	RMainCallback r_main_radare2;
 	// int (*r_main_radare2)(int argc, char **argv);
 	int (*r_main_rafind2)(int argc, const char **argv);
+	int (*r_main_r2pm)(int argc, const char **argv);
 	int (*r_main_radiff2)(int argc, const char **argv);
 	int (*r_main_rabin2)(int argc, const char **argv);
 	int (*r_main_rarun2)(int argc, const char **argv);
@@ -387,7 +408,7 @@ R_API int r_core_bind(RCore *core, RCoreBind *bnd);
 typedef struct r_core_cmpwatch_t {
 	ut64 addr;
 	int size;
-	char cmd[32];
+	char *cmd;
 	ut8 *odata;
 	ut8 *ndata;
 } RCoreCmpWatcher;
@@ -395,13 +416,21 @@ typedef struct r_core_cmpwatch_t {
 typedef int (*RCoreSearchCallback)(RCore *core, ut64 from, ut8 *buf, int len);
 
 #ifdef R_API
-//#define r_core_ncast(x) (RCore*)(size_t)(x)
+
+typedef int RCmdReturnCode;
+#define R_CMD_RC_FAILURE 1
+#define R_CMD_RC_SUCCESS 0
+#define R_CMD_RC_QUIT -2
+#define R_CMD_RC_FASTQUIT -1
+#define r_core_return_value(core, val) (core)->num->value = (val)
+#define r_core_return_code(core, val) (core)->rc = (val)
+
 R_API RList *r_core_list_themes(RCore *core);
 R_API char *r_core_get_theme(RCore *core);
 R_API const char *r_core_get_section_name(RCore *core, ut64 addr);
 R_API RCons *r_core_get_cons(RCore *core);
 R_API RBin *r_core_get_bin(RCore *core);
-R_API RConfig *r_core_get_config (RCore *core);
+R_API RConfig *r_core_get_config(RCore *core);
 R_API bool r_core_init(RCore *core);
 R_API void r_core_bind_cons(RCore *core); // to restore pointers in cons
 R_API RCore *r_core_new(void);
@@ -416,8 +445,8 @@ R_API void r_core_config_update(RCore *core);
 R_API void r_core_parse_radare2rc(RCore *r);
 R_API int r_core_prompt(RCore *core, int sync);
 R_API int r_core_prompt_exec(RCore *core);
-R_API int r_core_lines_initcache (RCore *core, ut64 start_addr, ut64 end_addr);
-R_API int r_core_lines_currline (RCore *core);
+R_API int r_core_lines_initcache(RCore *core, ut64 start_addr, ut64 end_addr);
+R_API int r_core_lines_currline(RCore *core);
 R_API bool r_core_prompt_loop(RCore *core);
 R_API ut64 r_core_pava(RCore *core, ut64 addr);
 R_API int r_core_cmd(RCore *core, const char *cmd, bool log);
@@ -427,35 +456,38 @@ R_API int r_core_fgets(char *buf, int len);
 R_API RFlagItem *r_core_flag_get_by_spaces(RFlag *f, ut64 off);
 R_API int r_core_cmdf(RCore *core, const char *fmt, ...) R_PRINTF_CHECK(2, 3);
 R_API int r_core_cmd0(RCore *core, const char *cmd);
+R_API void r_core_cmd_r(RCore *core, const char *cmd);
 R_API void r_core_cmd_queue(RCore *core, const char *line);
 R_API void r_core_cmd_queue_wait(RCore *core);
 R_API void r_core_cmd_init(RCore *core);
 R_API int r_core_cmd_pipe(RCore *core, char *radare_cmd, char *shell_cmd);
 R_API char *r_core_cmd_str(RCore *core, const char *cmd);
+R_API char *r_core_cmd_str_r(RCore *core, const char *cmd);
 R_API char *r_core_cmd_strf(RCore *core, const char *fmt, ...) R_PRINTF_CHECK(2, 3);
 R_API char *r_core_cmd_str_pipe(RCore *core, const char *cmd);
+R_API RBuffer *r_core_cmd_tobuf(RCore *core, const char *cmd);
 R_API int r_core_cmd_file(RCore *core, const char *file);
 R_API int r_core_cmd_lines(RCore *core, const char *lines);
 R_API int r_core_cmd_command(RCore *core, const char *command);
 R_API void r_core_af(RCore *core, ut64 addr, const char *name, bool anal_calls);
-R_API bool r_core_run_script (RCore *core, const char *file);
+R_API bool r_core_run_script(RCore *core, const char *file);
 R_API bool r_core_seek(RCore *core, ut64 addr, bool rb);
 R_API bool r_core_visual_bit_editor(RCore *core);
-R_API int r_core_seek_base (RCore *core, const char *hex);
-R_API void r_core_seek_previous (RCore *core, const char *type);
-R_API void r_core_seek_next (RCore *core, const char *type);
+R_API int r_core_seek_base(RCore *core, const char *hex);
+R_API void r_core_seek_previous(RCore *core, const char *type);
+R_API void r_core_seek_next(RCore *core, const char *type);
 R_API int r_core_seek_align(RCore *core, ut64 align, int count);
 R_API void r_core_arch_bits_at(RCore *core, ut64 addr, R_OUT R_NULLABLE int *bits, R_OUT R_BORROW R_NULLABLE const char **arch);
 R_API void r_core_seek_arch_bits(RCore *core, ut64 addr);
 R_API int r_core_block_read(RCore *core);
 R_API int r_core_block_size(RCore *core, int bsize);
 R_API int r_core_seek_size(RCore *core, ut64 addr, int bsize);
-R_API int r_core_is_valid_offset (RCore *core, ut64 offset);
+R_API int r_core_is_valid_offset(RCore *core, ut64 offset);
 R_API int r_core_shift_block(RCore *core, ut64 addr, ut64 b_size, st64 dist);
 R_API void r_core_autocomplete(R_NULLABLE RCore *core, RLineCompletion *completion, RLineBuffer *buf, RLinePromptType prompt_type);
 R_API void r_core_print_scrollbar(RCore *core);
 R_API void r_core_print_scrollbar_bottom(RCore *core);
-R_API void r_core_visual_prompt_input (RCore *core);
+R_API void r_core_visual_prompt_input(RCore *core);
 R_API void r_core_visual_toggle_decompiler_disasm(RCore *core, bool for_graph, bool reset);
 R_API void r_core_visual_applyDisMode(RCore *core, int disMode);
 R_API void r_core_visual_applyHexMode(RCore *core, int hexMode);
@@ -471,14 +503,14 @@ R_API int r_core_visual(RCore *core, const char *input);
 R_API int r_core_visual_graph(RCore *core, RAGraph *g, RAnalFunction *_fcn, int is_interactive);
 R_API void r_core_visual_browse(RCore *core, const char *arg);
 R_API int r_core_visual_cmd(RCore *core, const char *arg);
-R_API void r_core_visual_seek_animation (RCore *core, ut64 addr);
+R_API void r_core_visual_seek_animation(RCore *core, ut64 addr);
 R_API void r_core_visual_asm(RCore *core, ut64 addr);
 R_API void r_core_visual_colors(RCore *core);
 R_API int r_core_visual_xrefs_x(RCore *core);
 R_API int r_core_visual_xrefs_X(RCore *core);
 R_API void r_core_visual_showcursor(RCore *core, int x);
 R_API void r_core_visual_offset(RCore *core);
-R_API int r_core_visual_hud(RCore *core);
+R_API int r_core_visual_hud(RCore *core); // R2_580 bool
 R_API void r_core_visual_jump(RCore *core, ut8 ch);
 R_API void r_core_visual_disasm_up(RCore *core, int *cols);
 R_API void r_core_visual_disasm_down(RCore *core, RAsmOp *op, int *cols);
@@ -495,14 +527,15 @@ R_API bool r_core_panels_load(RCore *core, const char *_name);
 R_API void r_core_anal_type_init(RCore *core);
 R_API char *r_core_anal_hasrefs_to_depth(RCore *core, ut64 value, PJ *pj, int depth);
 R_API void r_core_link_stroff(RCore *core, RAnalFunction *fcn);
-R_API void r_core_anal_inflags (RCore *core, const char *glob);
-R_API bool cmd_anal_objc (RCore *core, const char *input, bool auto_anal);
+R_API void r_core_anal_inflags(RCore *core, const char *glob);
+R_API bool cmd_anal_objc(RCore *core, const char *input, bool auto_anal);
 R_API void r_core_anal_cc_init(RCore *core);
 R_API void r_core_anal_paths(RCore *core, ut64 from, ut64 to, bool followCalls, int followDepth, bool is_json);
 
 R_API void r_core_list_io(RCore *core);
-R_API RListInfo *r_listinfo_new (const char *name, RInterval pitv, RInterval vitv, int perm, const char *extra);
-R_API void r_listinfo_free (RListInfo *info);
+R_API RListInfo *r_listinfo_new(const char *name, RInterval pitv, RInterval vitv, int perm, const char *extra);
+R_API void r_listinfo_free(RListInfo *info);
+R_API void r_core_visual_slides(RCore *core, const char *file);
 /* visual marks */
 R_API void r_core_visual_mark_seek(RCore *core, ut8 ch);
 R_API void r_core_visual_mark(RCore *core, ut8 ch);
@@ -523,18 +556,19 @@ R_API RIODesc *r_core_file_open_many(RCore *r, const char *file, int flags, ut64
 R_API bool r_core_file_close_all_but(RCore *core);
 
 
-R_API int r_core_setup_debugger (RCore *r, const char *debugbackend, bool attach);
+R_API int r_core_setup_debugger(RCore *r, const char *debugbackend, bool attach);
 R_API int r_core_seek_delta(RCore *core, st64 addr);
 R_API bool r_core_extend_at(RCore *core, ut64 addr, int size);
 R_API bool r_core_write_at(RCore *core, ut64 addr, const ut8 *buf, int size);
 R_API int r_core_write_op(RCore *core, const char *arg, char op);
 R_API ut8* r_core_transform_op(RCore *core, const char *arg, char op);
-R_API int r_core_set_file_by_fd (RCore * core, ut64 bin_fd);
-R_API int r_core_set_file_by_name (RBin * bin, const char * name);
+R_API int r_core_set_file_by_fd(RCore *core, ut64 bin_fd);
+R_API int r_core_set_file_by_name(RBin * bin, const char *name);
 
-R_API void r_core_debug_rr (RCore *core, RReg *reg, int mode);
+R_API void r_core_debug_rr(RCore *core, RReg *reg, int mode);
 
 /* fortune */
+R_IPI RList *r_core_fortune_types(void);
 R_API void r_core_fortune_list_types(void);
 R_API void r_core_fortune_list(RCore *core);
 R_API void r_core_fortune_print_random(RCore *core);
@@ -551,17 +585,17 @@ R_API int r_core_yank(RCore *core, ut64 addr, int len);
 R_API int r_core_yank_string(RCore *core, ut64 addr, int maxlen);
 R_API bool r_core_yank_hexpair(RCore *core, const char *input);
 R_API int r_core_yank_paste(RCore *core, ut64 addr, int len);
-R_API int r_core_yank_set (RCore *core, ut64 addr, const ut8 *buf, ut32 len);  // set yank buffer bytes
-R_API int r_core_yank_set_str (RCore *core, ut64 addr, const char *buf, ut32 len); // Null terminate the bytes
+R_API int r_core_yank_set(RCore *core, ut64 addr, const ut8 *buf, ut32 len);  // set yank buffer bytes
+R_API int r_core_yank_set_str(RCore *core, ut64 addr, const char *buf, ut32 len); // Null terminate the bytes
 R_API int r_core_yank_to(RCore *core, const char *arg);
-R_API bool r_core_yank_dump (RCore *core, ut64 pos, int format);
-R_API int r_core_yank_hexdump (RCore *core, ut64 pos);
-R_API int r_core_yank_cat (RCore *core, ut64 pos);
-R_API int r_core_yank_cat_string (RCore *core, ut64 pos);
-R_API int r_core_yank_hud_file (RCore *core, const char *input);
-R_API int r_core_yank_hud_path (RCore *core, const char *input, int dir);
-R_API bool r_core_yank_file_ex (RCore *core, const char *input);
-R_API int r_core_yank_file_all (RCore *core, const char *input);
+R_API bool r_core_yank_dump(RCore *core, ut64 pos, int format);
+R_API int r_core_yank_hexdump(RCore *core, ut64 pos);
+R_API int r_core_yank_cat(RCore *core, ut64 pos);
+R_API int r_core_yank_cat_string(RCore *core, ut64 pos);
+R_API int r_core_yank_hud_file(RCore *core, const char *input);
+R_API int r_core_yank_hud_path(RCore *core, const char *input, int dir);
+R_API bool r_core_yank_file_ex(RCore *core, const char *input);
+R_API int r_core_yank_file_all(RCore *core, const char *input);
 
 #define R_CORE_LOADLIBS_ENV 1
 #define R_CORE_LOADLIBS_HOME 2
@@ -572,9 +606,6 @@ R_API int r_core_yank_file_all (RCore *core, const char *input);
 R_API void r_core_loadlibs_init(RCore *core);
 R_API bool r_core_loadlibs(RCore *core, int where, const char *path);
 R_API int r_core_cmd_buffer(RCore *core, const char *buf);
-R_API int r_core_cmdf(RCore *core, const char *fmt, ...) R_PRINTF_CHECK(2, 3);
-R_API int r_core_cmd0(RCore *core, const char *cmd);
-R_API char *r_core_cmd_str(RCore *core, const char *cmd);
 R_API int r_core_cmd_foreach(RCore *core, const char *cmd, char *each);
 R_API int r_core_cmd_foreach3(RCore *core, const char *cmd, char *each);
 R_API char *r_core_op_str(RCore *core, ut64 addr);
@@ -585,7 +616,7 @@ R_API char *r_core_disassemble_bytes(RCore *core, ut64 addr, int b);
 /* carg.c */
 R_API RList *r_core_get_func_args(RCore *core, const char *func_name);
 R_API void r_core_print_func_args(RCore *core);
-R_API char *resolve_fcn_name(RAnal *anal, const char * func_name);
+R_API char *resolve_fcn_name(RAnal *anal, const char *func_name);
 R_API int r_core_get_stacksz(RCore *core, ut64 from, ut64 to);
 
 /* anal.c */
@@ -593,8 +624,8 @@ R_API RAnalOp* r_core_anal_op(RCore *core, ut64 addr, int mask);
 R_IPI int core_type_by_addr(RCore *core, ut64 addr);
 R_API void r_core_anal_esil(RCore *core, const char *str, const char *addr);
 R_API void r_core_anal_fcn_merge(RCore *core, ut64 addr, ut64 addr2);
-R_API const char *r_core_anal_optype_colorfor(RCore *core, ut64 addr, bool verbose);
-R_API ut64 r_core_anal_address (RCore *core, ut64 addr);
+R_API const char *r_core_anal_optype_colorfor(RCore *core, ut64 addr, ut8 ch, bool verbose);
+R_API ut64 r_core_anal_address(RCore *core, ut64 addr);
 R_API void r_core_anal_undefine(RCore *core, ut64 off);
 R_API void r_core_anal_hint_print(RAnal* a, ut64 addr, int mode);
 R_API void r_core_anal_hint_list(RAnal *a, int mode);
@@ -610,7 +641,7 @@ R_API int r_core_anal_refs(RCore *core, const char *input);
 R_API void r_core_agraph_print(RCore *core, int use_utf, const char *input);
 R_API bool r_core_esil_cmd(RAnalEsil *esil, const char *cmd, ut64 a1, ut64 a2);
 R_API int r_core_esil_step(RCore *core, ut64 until_addr, const char *until_expr, ut64 *prev_addr, bool stepOver);
-R_API int r_core_esil_step_back(RCore *core);
+R_API bool r_core_esil_step_back(RCore *core);
 R_API ut64 r_core_anal_get_bbaddr(RCore *core, ut64 addr);
 R_API bool r_core_anal_bb_seek(RCore *core, ut64 addr);
 R_API bool r_core_anal_fcn(RCore *core, ut64 at, ut64 from, int reftype, int depth);
@@ -629,8 +660,8 @@ R_API int r_core_anal_graph_fcn(RCore *core, char *input, int opts);
 R_API RList* r_core_anal_graph_to(RCore *core, ut64 addr, int n);
 R_API int r_core_anal_ref_list(RCore *core, int rad);
 R_API int r_core_anal_all(RCore *core);
-R_API RList* r_core_anal_cycles (RCore *core, int ccl);
-R_API RList *r_core_anal_fcn_get_calls (RCore *core, RAnalFunction *fcn); // get all calls from a function
+R_API RList* r_core_anal_cycles(RCore *core, int ccl);
+R_API RList *r_core_anal_fcn_get_calls(RCore *core, RAnalFunction *fcn); // get all calls from a function
 
 /*tp.c*/
 R_API void r_core_anal_type_match(RCore *core, RAnalFunction *fcn);
@@ -648,23 +679,31 @@ typedef struct r_core_asm_hit {
 	ut8 valid;
 } RCoreAsmHit;
 
-R_API RBuffer *r_core_syscall (RCore *core, const char *name, const char *args);
-R_API RBuffer *r_core_syscallf (RCore *core, const char *name, const char *fmt, ...) R_PRINTF_CHECK(3, 4);
+R_API RBuffer *r_core_syscall(RCore *core, const char *name, const char *args);
+R_API RBuffer *r_core_syscallf(RCore *core, const char *name, const char *fmt, ...) R_PRINTF_CHECK(3, 4);
 R_API RCoreAsmHit *r_core_asm_hit_new(void);
 R_API RList *r_core_asm_hit_list_new(void);
 R_API void r_core_asm_hit_free(void *_hit);
 R_API void r_core_set_asm_configs(RCore *core, char *arch, ut32 bits, int segoff);
 R_API char* r_core_asm_search(RCore *core, const char *input);
 R_API RList *r_core_asm_strsearch(RCore *core, const char *input, ut64 from, ut64 to, int maxhits, int regexp, int everyByte, int mode);
-R_API RList *r_core_asm_bwdisassemble (RCore *core, ut64 addr, int n, int len);
-R_API RList *r_core_asm_back_disassemble_instr (RCore *core, ut64 addr, int len, ut32 hit_count, ut32 extra_padding);
-R_API RList *r_core_asm_back_disassemble_byte (RCore *core, ut64 addr, int len, ut32 hit_count, ut32 extra_padding);
-R_API ut32 r_core_asm_bwdis_len (RCore* core, int* len, ut64* start_addr, ut32 l);
-R_API int r_core_print_disasm(RCore *core, ut64 addr, ut8 *buf, int len, int count, bool count_bytes, bool read_buffer_only, bool json, PJ *pj, RAnalFunction *pdf);
+R_API RList *r_core_asm_bwdisassemble(RCore *core, ut64 addr, int n, int len);
+R_API RList *r_core_asm_back_disassemble_instr(RCore *core, ut64 addr, int len, ut32 hit_count, ut32 extra_padding);
+R_API RList *r_core_asm_back_disassemble_byte(RCore *core, ut64 addr, int len, ut32 hit_count, ut32 extra_padding);
+R_API ut32 r_core_asm_bwdis_len(RCore* core, int* len, ut64* start_addr, ut32 l);
+
+
+enum pdu_condition {
+	//esil,
+	instruction,
+	opcode
+};
+R_API int r_core_print_disasm(RCore *core, ut64 addr, ut8 *buf, int len, int count, enum pdu_condition pdu_condition_type, const void *pdu_condition, bool count_bytes, bool json, PJ *pj, RAnalFunction *pdf);
 R_API int r_core_print_disasm_json(RCore *core, ut64 addr, ut8 *buf, int len, int lines, PJ *pj);
 R_API int r_core_print_disasm_instructions_with_buf(RCore *core, ut64 address, ut8 *buf, int nb_bytes, int nb_opcodes);
 R_API int r_core_print_disasm_instructions(RCore *core, int nb_bytes, int nb_opcodes);
 R_API int r_core_print_disasm_all(RCore *core, ut64 addr, int l, int len, int mode);
+
 R_API int r_core_disasm_pdi_with_buf(RCore *core, ut64 address, ut8 *buf, ut32 nb_opcodes, ut32 nb_bytes, int fmt);
 R_API int r_core_disasm_pdi(RCore *core, int nb_opcodes, int nb_bytes, int fmt);
 R_API int r_core_disasm_pde(RCore *core, int nb_opcodes, int mode);
@@ -674,23 +713,22 @@ R_API int r_core_flag_in_middle(RCore *core, ut64 at, int oplen, int *midflags);
 R_API int r_core_bb_starts_in_middle(RCore *core, ut64 at, int oplen);
 
 // both do the same, we should get rid of one of them
-R_API bool r_core_bin_raise (RCore *core, ut32 bfid);
+R_API bool r_core_bin_raise(RCore *core, ut32 bfid);
 R_API bool r_core_bin_set_cur(RCore *core, RBinFile *binfile);
 
-R_API int r_core_bin_set_env (RCore *r, RBinFile *binfile);
-R_API int r_core_bin_set_by_fd (RCore *core, ut64 bin_fd);
-R_API int r_core_bin_set_by_name (RCore *core, const char *name);
-R_API int r_core_bin_reload(RCore *core, const char *file, ut64 baseaddr);
+R_API bool r_core_bin_set_env(RCore *r, RBinFile *binfile);
+R_API bool r_core_bin_set_by_fd(RCore *core, ut64 bin_fd);
+R_API bool r_core_bin_set_by_name(RCore *core, const char *name);
 R_API bool r_core_bin_load(RCore *core, const char *file, ut64 baseaddr);
-R_API int r_core_bin_rebase(RCore *core, ut64 baddr);
+R_API bool r_core_bin_rebase(RCore *core, ut64 baddr);
 R_API void r_core_bin_export_info(RCore *core, int mode);
-R_API int r_core_bin_list(RCore *core, int mode);
-R_API bool r_core_bin_delete (RCore *core, ut32 binfile_idx);
+R_API bool r_core_bin_list(RCore *core, int mode);
+R_API bool r_core_bin_delete(RCore *core, ut32 binfile_idx);
 R_API ut64 r_core_bin_impaddr(RBin *bin, int va, const char *name);
 
 // XXX - this is kinda hacky, maybe there should be a way to
 // refresh the bin environment without specific calls?
-R_API int r_core_pseudo_code (RCore *core, const char *input);
+R_API int r_core_pseudo_code(RCore *core, const char *input);
 
 /* gdiff.c */
 R_API int r_core_zdiff(RCore *c, RCore *c2);
@@ -698,14 +736,14 @@ R_API int r_core_gdiff(RCore *core1, RCore *core2);
 R_API int r_core_gdiff_fcn(RCore *c, ut64 addr, ut64 addr2);
 
 R_API bool r_core_project_open(RCore *core, const char *file);
-R_API int r_core_project_cat(RCore *core, const char *name);
+R_API void r_core_project_cat(RCore *core, const char *name);
 R_API int r_core_project_delete(RCore *core, const char *prjfile);
 R_API int r_core_project_list(RCore *core, int mode);
 R_API bool r_core_project_save_script(RCore *core, const char *file, int opts);
 R_API bool r_core_project_save(RCore *core, const char *file);
 R_API char *r_core_project_name(RCore *core, const char *file);
-R_API char *r_core_project_notes_file (RCore *core, const char *file);
-
+R_API char *r_core_project_notes_file(RCore *core, const char *file);
+R_API void r_core_project_undirty(RCore *core);
 R_API char *r_core_sysenv_begin(RCore *core, const char *cmd);
 R_API void r_core_sysenv_end(RCore *core, const char *cmd);
 
@@ -760,15 +798,15 @@ typedef struct r_core_bin_filter_t {
 	const char *name;
 } RCoreBinFilter;
 
-R_API int r_core_bin_info (RCore *core, int action, PJ *pj, int mode, int va, RCoreBinFilter *filter, const char *chksum);
-R_API int r_core_bin_set_arch_bits (RCore *r, const char *name, const char * arch, ut16 bits);
-R_API int r_core_bin_update_arch_bits (RCore *r);
+R_API bool r_core_bin_info(RCore *core, int action, PJ *pj, int mode, int va, RCoreBinFilter *filter, const char *chksum);
+R_API bool r_core_bin_set_arch_bits(RCore *r, const char *name, const char *arch, ut16 bits);
+R_API bool r_core_bin_update_arch_bits(RCore *r);
 R_API char *r_core_bin_method_flags_str(ut64 flags, int mode);
 R_API bool r_core_pdb_info(RCore *core, const char *file, PJ *pj, int mode);
 
 /* rtr */
-R_API int r_core_rtr_cmds (RCore *core, const char *port);
-R_API char *r_core_rtr_cmds_query (RCore *core, const char *host, const char *port, const char *cmd);
+R_API int r_core_rtr_cmds(RCore *core, const char *port);
+R_API char *r_core_rtr_cmds_query(RCore *core, const char *host, const char *port, const char *cmd);
 R_API void r_core_rtr_help(RCore *core);
 R_API void r_core_rtr_pushout(RCore *core, const char *input);
 R_API void r_core_rtr_list(RCore *core);
@@ -795,27 +833,28 @@ R_API int r_core_visual_view_zigns(RCore *core);
 R_API int r_core_visual_view_rop(RCore *core);
 R_API int r_core_visual_comments(RCore *core);
 R_API int r_core_visual_prompt(RCore *core);
-R_API bool r_core_visual_esil (RCore *core, const char *input);
+R_API bool r_core_visual_esil(RCore *core, const char *input);
 R_API int r_core_search_preludes(RCore *core, bool log);
 R_API int r_core_search_prelude(RCore *core, ut64 from, ut64 to, const ut8 *buf, int blen, const ut8 *mask, int mlen);
-R_API RList* /*<RIOMap*>*/ r_core_get_boundaries_prot (RCore *core, int protection, const char *mode, const char *prefix);
+R_API RList* /*<RIOMap*>*/ r_core_get_boundaries_prot(RCore *core, int protection, const char *mode, const char *prefix);
 
-R_API int r_core_patch (RCore *core, const char *patch);
+R_API int r_core_patch(RCore *core, const char *patch);
 
 R_API void r_core_hack_help(const RCore *core);
 R_API int r_core_hack(RCore *core, const char *op);
 R_API bool r_core_dump(RCore *core, const char *file, ut64 addr, ut64 size, int append);
 R_API void r_core_diff_show(RCore *core, RCore *core2);
+R_API void r_core_diff_show_json(RCore *core, RCore *core2);
 R_API void r_core_clippy(RCore *core, const char *msg);
 
 /* watchers */
-R_API void r_core_cmpwatch_free (RCoreCmpWatcher *w);
-R_API RCoreCmpWatcher *r_core_cmpwatch_get (RCore *core, ut64 addr);
-R_API int r_core_cmpwatch_add (RCore *core, ut64 addr, int size, const char *cmd);
-R_API int r_core_cmpwatch_del (RCore *core, ut64 addr);
-R_API int r_core_cmpwatch_update (RCore *core, ut64 addr);
-R_API int r_core_cmpwatch_show (RCore *core, ut64 addr, int mode);
-R_API int r_core_cmpwatch_revert (RCore *core, ut64 addr);
+R_API void r_core_cmpwatch_free(RCoreCmpWatcher *w);
+R_API R_BORROW RCoreCmpWatcher *r_core_cmpwatch_get(RCore *core, ut64 addr);
+R_API bool r_core_cmpwatch_add(RCore *core, ut64 addr, int size, const char *cmd);
+R_API bool r_core_cmpwatch_del(RCore *core, ut64 addr);
+R_API bool r_core_cmpwatch_update(RCore *core, ut64 addr);
+R_API bool r_core_cmpwatch_show(RCore *core, ut64 addr, int mode);
+R_API bool r_core_cmpwatch_revert(RCore *core, ut64 addr);
 
 /* undo */
 R_API RCoreUndo *r_core_undo_new(ut64 offset, const char *action, const char *revert);
@@ -823,13 +862,17 @@ R_API void r_core_undo_print(RCore *core, int mode, RCoreUndoCondition *cond);
 R_API void r_core_undo_free(RCoreUndo *cu);
 R_API void r_core_undo_push(RCore *core, RCoreUndo *cu);
 R_API void r_core_undo_pop(RCore *core);
+#if R2_580
+R_API void r_core_undo_up(RCore *core);
+R_API void r_core_undo_down(RCore *core);
+#endif
 
 /* logs */
 typedef int (*RCoreLogCallback)(RCore *core, int count, const char *message);
 R_API void r_core_log_free(RCoreLog *log);
-R_API void r_core_log_init (RCoreLog *log);
+R_API void r_core_log_init(RCoreLog *log);
 R_API char *r_core_log_get(RCore *core, int index);
-R_API RCoreLog *r_core_log_new (void);
+R_API RCoreLog *r_core_log_new(void);
 R_API bool r_core_log_run(RCore *core, const char *buf, RCoreLogCallback cb);
 R_API int r_core_log_list(RCore *core, int n, int count, char fmt);
 R_API void r_core_log_add(RCore *core, const char *msg);
@@ -840,7 +883,10 @@ typedef char *(*PrintItemCallback)(void *user, void *p, bool selected);
 R_API char *r_str_widget_list(void *user, RList *list, int rows, int cur, PrintItemCallback cb);
 R_API PJ *r_core_pj_new(RCore *core);
 /* help */
-R_API void r_core_cmd_help(const RCore *core, const char * help[]);
+R_API void r_core_cmd_help(const RCore *core, const char *help[]);
+R_API void r_core_cmd_help_json(const RCore *core, const char *help[]);
+R_API void r_core_cmd_help_match(const RCore *core, const char *help[], R_BORROW R_NONNULL char *cmd, bool exact);
+R_API void r_core_cmd_help_match_spec(const RCore *core, const char *help[], R_BORROW R_NONNULL char *cmd, char spec, bool exact);
 
 /* anal stats */
 
@@ -860,11 +906,11 @@ typedef struct {
 } RCoreAnalStats;
 
 R_API bool core_anal_bbs(RCore *core, const char* input);
-R_API bool core_anal_bbs_range (RCore *core, const char* input);
+R_API bool core_anal_bbs_range(RCore *core, const char* input);
 R_API char *r_core_anal_hasrefs(RCore *core, ut64 value, int mode);
 R_API char *r_core_anal_get_comments(RCore *core, ut64 addr);
-R_API RCoreAnalStats* r_core_anal_get_stats (RCore *a, ut64 from, ut64 to, ut64 step);
-R_API void r_core_anal_stats_free (RCoreAnalStats *s);
+R_API RCoreAnalStats* r_core_anal_get_stats(RCore *a, ut64 from, ut64 to, ut64 step);
+R_API void r_core_anal_stats_free(RCoreAnalStats *s);
 
 R_API void r_core_syscmd_ls(const char *input);
 R_API void r_core_syscmd_cat(const char *file);
@@ -914,8 +960,8 @@ typedef void (*RCoreTaskOneShot)(void *);
 R_API void r_core_echo(RCore *core, const char *msg);
 R_API RTable *r_core_table(RCore *core, const char *name);
 
-R_API void r_core_task_scheduler_init (RCoreTaskScheduler *tasks, RCore *core);
-R_API void r_core_task_scheduler_fini (RCoreTaskScheduler *tasks);
+R_API void r_core_task_scheduler_init(RCoreTaskScheduler *tasks, RCore *core);
+R_API void r_core_task_scheduler_fini(RCoreTaskScheduler *tasks);
 R_API RCoreTask *r_core_task_get(RCoreTaskScheduler *scheduler, int id);
 R_API RCoreTask *r_core_task_get_incref(RCoreTaskScheduler *scheduler, int id);
 R_API void r_core_task_print(RCore *core, RCoreTask *task, PJ *pj, int mode);
@@ -940,12 +986,11 @@ R_API void r_core_task_del_all_done(RCoreTaskScheduler *scheduler);
 R_API RCoreTask *r_core_task_self(RCoreTaskScheduler *scheduler);
 R_API void r_core_task_join(RCoreTaskScheduler *scheduler, RCoreTask *current, int id);
 typedef void (*inRangeCb) (RCore *core, ut64 from, ut64 to, int vsize, void *cb_user);
-R_API int r_core_search_value_in_range (RCore *core, RInterval search_itv,
-		ut64 vmin, ut64 vmax, int vsize, inRangeCb cb, void *cb_user);
+R_IPI int r_core_search_value_in_range(RCore *core, bool relative, RInterval search_itv, ut64 vmin, ut64 vmax, int vsize, inRangeCb cb, void *cb_user);
 
 R_API RCoreAutocomplete *r_core_autocomplete_add(RCoreAutocomplete *parent, const char* cmd, int type, bool lock);
 R_API void r_core_autocomplete_free(RCoreAutocomplete *obj);
-R_API void r_core_autocomplete_reload (RCore *core);
+R_API void r_core_autocomplete_reload(RCore *core);
 R_API RCoreAutocomplete *r_core_autocomplete_find(RCoreAutocomplete *parent, const char* cmd, bool exact);
 R_API bool r_core_autocomplete_remove(RCoreAutocomplete *parent, const char* cmd);
 R_API void r_core_anal_propagate_noreturn(RCore *core, ut64 addr);
@@ -957,40 +1002,7 @@ extern RCorePlugin r_core_plugin_sixref;
 R_API bool r_core_plugin_init(RCmd *cmd);
 R_API bool r_core_plugin_add(RCmd *cmd, RCorePlugin *plugin);
 R_API bool r_core_plugin_check(RCmd *cmd, const char *a0);
-R_API bool r_core_plugin_fini(RCmd *cmd);
-
-
-/* DECOMPILER PRINTING FUNCTIONS */
-/**
- * @brief Prints the data contained in the specified RAnnotatedCode in JSON format.
- *
- * The function will print the output in console using the function r_cons_printf();
- *
- * @param code Pointer to a RAnnotatedCode.
- */
-R_API void r_codemeta_print_json(RCodeMeta *code);
-/**
- * @brief Prints the decompiled code from the specified RAnnotatedCode.
- *
- * This function is used for printing the output of commands pdg and pdgo.
- * It can print the decompiled code with or without offsets. If line_offsets is a null pointer,
- * the output will be printed without offsets (pdg), otherwise, the output will be
- * printed with offsets.
- * This function will print the output in console using the function r_cons_printf();
- *
- * @param code Pointer to a RAnnotatedCode.
- * @param line_offsets Pointer to a @ref RVector that contains offsets for the decompiled code.
- */
-R_API void r_codemeta_print(RCodeMeta *code, RVector *line_offsets);
-/**
- * @brief  Prints the decompiled code as comments
- *
- * This function is used for the output of command pdg*
- * Output will be printed in console using the function r_cons_printf();
- *
- * @param code Pointer to a RAnnotatedCode.
- */
-R_API void r_codemeta_print_comment_cmds(RCodeMeta *code);
+R_API void r_core_plugin_fini(RCmd *cmd);
 
 #endif
 
