@@ -18,6 +18,9 @@ typedef SHA_CTX R_SHA_CTX;
 typedef SHA256_CTX R_SHA256_CTX;
 typedef SHA512_CTX R_SHA384_CTX;
 typedef SHA512_CTX R_SHA512_CTX;
+#define SHA256_BLOCK_LENGTH SHA256_CBLOCK
+#define SHA384_BLOCK_LENGTH SHA384_CBLOCK
+#define SHA512_BLOCK_LENGTH SHA512_CBLOCK
 #else
 #define MD5_CTX R_MD5_CTX
 
@@ -54,7 +57,7 @@ typedef R_SHA512_CTX R_SHA384_CTX;
 
 
 /*
- * Since we have not enought space in bitmask, you may do fine
+ * Since we have not enough space in bitmask, you may do fine
  * selection of required hash functions by the followed macros.
  *
  * TODO: subject to place in config
@@ -77,6 +80,11 @@ typedef ut32 utcrc;
 #endif
 #define UTCRC_C(x) ((utcrc)(x))
 
+R_API ut8 r_hash_fletcher8(const ut8 *d, size_t length);
+R_API ut16 r_hash_fletcher16(const ut8 *data, size_t len);
+R_API ut32 r_hash_fletcher32(const ut8 *data, size_t len);
+R_API ut64 r_hash_fletcher64(const ut8 *addr, size_t len);
+
 typedef struct {
 	utcrc crc;
 	ut32 size;
@@ -171,8 +179,8 @@ struct r_hash_t {
 	R_SHA384_CTX sha384;
 	R_SHA512_CTX sha512;
 	bool rst;
-	ut8 digest[128];
 	double entropy;
+	ut8 R_ALIGNED(8) digest[128];
 };
 
 typedef struct r_hash_seed_t {
@@ -181,6 +189,7 @@ typedef struct r_hash_seed_t {
 	int len;
 } RHashSeed;
 
+#define R_HASH_SIZE_SSDEEP 128
 #define R_HASH_SIZE_CRC8_SMBUS 1
 #if R_HAVE_CRC8_EXTRA
 #define R_HASH_SIZE_CRC8_CDMA2000 1
@@ -269,8 +278,12 @@ typedef struct r_hash_seed_t {
 #define R_HASH_SIZE_XORPAIR 2
 #define R_HASH_SIZE_HAMDIST 1
 #define R_HASH_SIZE_LUHN 1
+#define R_HASH_SIZE_FLETCHER8 1
+#define R_HASH_SIZE_FLETCHER16 2
+#define R_HASH_SIZE_FLETCHER32 4
+#define R_HASH_SIZE_FLETCHER64 8
 
-#define R_HASH_NBITS (8*sizeof(ut64))
+#define R_HASH_NBITS (8*sizeof (ut64))
 
 enum HASH_INDICES {
 	R_HASH_IDX_MD5 = 0,
@@ -292,6 +305,7 @@ enum HASH_INDICES {
 	R_HASH_IDX_BASE91,
 	R_HASH_IDX_PUNYCODE,
 	R_HASH_IDX_LUHN,
+	R_HASH_IDX_SSDEEP,
 
 	R_HASH_IDX_CRC8_SMBUS,
 #if R_HAVE_CRC8_EXTRA
@@ -364,6 +378,10 @@ enum HASH_INDICES {
 	R_HASH_IDX_CRC64_ISO,
 #endif /* #if R_HAVE_CRC64_EXTRA */
 
+	R_HASH_IDX_FLETCHER8,
+	R_HASH_IDX_FLETCHER16,
+	R_HASH_IDX_FLETCHER32,
+	R_HASH_IDX_FLETCHER64,
 	R_HASH_NUM_INDICES
 };
 
@@ -387,6 +405,11 @@ enum HASH_INDICES {
 #define R_HASH_BASE91 (1ULL << R_HASH_IDX_BASE91)
 #define R_HASH_PUNYCODE (1ULL << R_HASH_IDX_PUNYCODE)
 #define R_HASH_LUHN (1ULL << R_HASH_IDX_LUHN)
+#define R_HASH_SSDEEP (1ULL << R_HASH_IDX_SSDEEP)
+#define R_HASH_FLETCHER8 (1ULL << R_HASH_IDX_FLETCHER8)
+#define R_HASH_FLETCHER16 (1ULL << R_HASH_IDX_FLETCHER16)
+#define R_HASH_FLETCHER32 (1ULL << R_HASH_IDX_FLETCHER32)
+#define R_HASH_FLETCHER64 (1ULL << R_HASH_IDX_FLETCHER64)
 
 #define R_HASH_CRC8_SMBUS (1ULL << R_HASH_IDX_CRC8_SMBUS)
 #if R_HAVE_CRC8_EXTRA
@@ -468,11 +491,13 @@ R_API void r_hash_free(RHash *ctx);
 
 /* methods */
 R_API ut8 *r_hash_do_md4(RHash *ctx, const ut8 *input, int len);
+R_API ut8 *r_hash_do_ssdeep(RHash *ctx, const ut8 *input, int len);
 R_API ut8 *r_hash_do_md5(RHash *ctx, const ut8 *input, int len);
 R_API ut8 *r_hash_do_sha1(RHash *ctx, const ut8 *input, int len);
 R_API ut8 *r_hash_do_sha256(RHash *ctx, const ut8 *input, int len);
 R_API ut8 *r_hash_do_sha384(RHash *ctx, const ut8 *input, int len);
 R_API ut8 *r_hash_do_sha512(RHash *ctx, const ut8 *input, int len);
+R_API ut8 *r_hash_do_hmac_sha256(RHash *ctx, const ut8 *input, int len, const ut8 *key, int klen);
 
 R_API char *r_hash_to_string(RHash *ctx, const char *name, const ut8 *data, int len);
 
@@ -493,7 +518,8 @@ R_API ut16 r_hash_xorpair(const ut8 *a, ut64 len);
 R_API int r_hash_parity(const ut8 *buf, ut64 len);
 R_API ut8 r_hash_mod255(const ut8 *b, ut64 len);
 R_API ut64 r_hash_luhn(const ut8 *buf, ut64 len);
-R_API utcrc r_hash_crc_preset (const ut8 *data, ut32 size, enum CRC_PRESETS preset);
+R_API char *r_hash_ssdeep(const ut8 *buf, size_t len);
+R_API utcrc r_hash_crc_preset(const ut8 *data, ut32 size, enum CRC_PRESETS preset);
 
 /* analysis */
 R_API ut8  r_hash_hamdist(const ut8 *buf, int len);
