@@ -1,14 +1,14 @@
 #ifndef R2_SOCKET_H
 #define R2_SOCKET_H
 
-#include "r_types.h"
-#include "r_bind.h"
-#include "r_list.h"
-
 #ifdef __cplusplus
 extern "C" {
 #endif
 
+#include "r_types.h"
+#include "r_bind.h"
+#include "r_list.h"
+
 R_LIB_VERSION_HEADER (r_socket);
 
 #if __UNIX__
@@ -16,10 +16,12 @@ R_LIB_VERSION_HEADER (r_socket);
 #include <sys/un.h>
 #include <poll.h>
 #include <arpa/inet.h>
+#ifndef __wasi__
 #include <netdb.h>
-#include <sys/socket.h>
 #include <sys/wait.h>
 #endif
+#include <sys/socket.h>
+#endif
 
 #if HAVE_LIB_SSL
 #include <openssl/ssl.h>
@@ -47,10 +49,11 @@ R_LIB_VERSION_HEADER (r_socket);
 #endif
 
 typedef struct {
-	int child;
 #if __WINDOWS__
 	HANDLE pipe;
+	HANDLE child;
 #else
+	int child;
 	int input[2];
 	int output[2];
 #endif
@@ -84,6 +87,7 @@ typedef struct r_socket_http_options {
 
 #define R_SOCKET_PROTO_TCP IPPROTO_TCP
 #define R_SOCKET_PROTO_UDP IPPROTO_UDP
+#define R_SOCKET_PROTO_CAN 0xc42b05
 #define R_SOCKET_PROTO_UNIX 0x1337
 #define R_SOCKET_PROTO_NONE 0
 #define R_SOCKET_PROTO_DEFAULT R_SOCKET_PROTO_TCP
@@ -112,7 +116,7 @@ R_API bool r_socket_block_time(RSocket *s, bool block, int sec, int usec);
 R_API int r_socket_flush(RSocket *s);
 R_API int r_socket_ready(RSocket *s, int secs, int usecs);
 R_API char *r_socket_to_string(RSocket *s);
-R_API int r_socket_write(RSocket *s, void *buf, int len);
+R_API int r_socket_write(RSocket *s, const void *buf, int len);
 R_API int r_socket_puts(RSocket *s, char *buf);
 R_API void r_socket_printf(RSocket *s, const char *fmt, ...) R_PRINTF_CHECK(2, 3);
 R_API int r_socket_read(RSocket *s, ut8 *read, int len);
@@ -157,6 +161,7 @@ R_API RSocketHTTPRequest *r_socket_http_accept(RSocket *s, RSocketHTTPOptions *s
 R_API void r_socket_http_response(RSocketHTTPRequest *rs, int code, const char *out, int x, const char *headers);
 R_API void r_socket_http_close(RSocketHTTPRequest *rs);
 R_API ut8 *r_socket_http_handle_upload(const ut8 *str, int len, int *olen);
+R_API void r_socket_http_free(RSocketHTTPRequest *rs);
 
 typedef int (*rap_server_open)(void *user, const char *file, int flg, int mode);
 typedef int (*rap_server_seek)(void *user, ut64 offset, int whence);
@@ -224,12 +229,21 @@ typedef struct r_run_profile_t {
 	char *_libpath;
 	char *_preload;
 	int _bits;
+	bool _time;
 	int _pid;
 	char *_pidfile;
+#if R2_580
+	// TODO more bools
+	bool _r2preload;
+	bool _docore;
+	bool _dofork;
+	/// OKK dodebug is no longer used
+#else
 	int _r2preload;
 	int _docore;
 	int _dofork;
 	int _dodebug;
+#endif
 	int _aslr;
 	int _maxstack;
 	int _maxproc;
