@@ -6,13 +6,18 @@
 #include <stdlib.h>
 #include <string.h>
 #include <stdbool.h>
+#include <stdint.h>
 #include <inttypes.h>
 #include <stdio.h>
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 #undef eprintf
 #define eprintf(...) fprintf(stderr,__VA_ARGS__)
 
-// Copied from https://gcc.gnu.org/wiki/Visibility
+// Inspired in https://gcc.gnu.org/wiki/Visibility
 #ifndef SDB_API
 	#undef SDB_IPI
 	#if defined _WIN32 || defined __CYGWIN__
@@ -37,7 +42,7 @@
 #define __MINGW__ 1
 #endif
 
-#if __WIN32__ || __MINGW__ || __WINDOWS__ || _MSC_VER
+#if defined __WIN32__ || __MINGW__ > 0 || __WINDOWS__ > 0 || _MSC_VER > 0
 #define __SDB_WINDOWS__ 1
 #undef DIRSEP
 #define DIRSEP '\\'
@@ -45,7 +50,11 @@
 #define lseek _lseek
 #include <windows.h>
 #include <io.h>
+#if __MINGW32__
+#define ULLFMT PRIx64
+#else
 #define ULLFMT "I64"
+#endif
 #undef HAVE_MMAN
 #define HAVE_MMAN 0
 #else
@@ -56,10 +65,10 @@
 #include <unistd.h>
 #undef HAVE_MMAN
 #define HAVE_MMAN 1
-#define ULLFMT "ll"
+#define ULLFMT PRIx64
 #endif
 
-#if __wasi__
+#if __wasi__ || EMSCRIPTEN
 #undef HAVE_MMAN
 #define HAVE_MMAN 0
 #endif
@@ -79,11 +88,11 @@
 #endif
 
 #ifndef ut8
-#define ut8 unsigned char
-#define ut32 unsigned int
-#define ut64 unsigned long long
-#define st64 long long
-#define boolt int
+#define ut8 uint8_t
+#define ut32 uint32_t
+#define ut64 uint64_t
+#define st64 int64_t
+
 // TODO: deprecate R_NEW
 #ifndef R_NEW
 //it means we are within sdb
@@ -144,4 +153,8 @@ static inline void ut32_unpack(char s[4], ut32 *u) {
 	*u = result;
 }
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif
